package com.tuigai.tuigai_tongcheng_api.mybatis;

import cn.hutool.core.util.ReflectUtil;

import com.tuigai.tuigai_tongcheng_api.utils.Utils;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.binding.MapperMethod;
import org.apache.ibatis.executor.parameter.ParameterHandler;
import org.apache.ibatis.executor.resultset.ResultSetHandler;
import org.apache.ibatis.executor.statement.StatementHandler;
import org.apache.ibatis.mapping.BoundSql;
import org.apache.ibatis.plugin.*;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.lang.reflect.Field;
import java.sql.PreparedStatement;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Objects;
import java.util.Properties;

@Component
@Intercepts({
        //@Signature(type = StatementHandler.class, method = "prepare", args = {Connection.class, Integer.class}),
        @Signature(type = ParameterHandler.class, method = "setParameters", args = PreparedStatement.class),
        @Signature(type = ResultSetHandler.class, method = "handleResultSets", args = {Statement.class})
})
@Slf4j
public class MyBatisInterceptor implements Interceptor {

    @Override
    public Object intercept(Invocation invocation) throws Throwable {
        Object target = invocation.getTarget();
        //拦截sql结果处理器
        if (target instanceof ResultSetHandler) {
            return resultDecrypt(invocation);
        }
        //拦截sql参数处理器
        if (target instanceof ParameterHandler) {
            return parameterEncrypt(invocation);
        }
        /*//拦截sql语句处理器
        if (target instanceof StatementHandler) {
            return replaceSql(invocation);
        }*/
        return invocation.proceed();
    }

    /**
     * 对mybatis映射结果进行字段解密
     *
     * @param invocation 参数
     * @return 结果
     * @throws Throwable 异常
     */
    private Object resultDecrypt(Invocation invocation) throws Throwable {

        //取出查询的结果
        Object resultObject = invocation.proceed();
        if (Objects.isNull(resultObject)) {
            return null;
        }
        //基于selectList
        if (resultObject instanceof ArrayList) {
            ArrayList resultList = (ArrayList) resultObject;
            if (CollectionUtils.isEmpty(resultList)) {
                return resultObject;
            }
            for (Object result : resultList) {
                if (needToDecrypt(result)) {
                    //逐一解密
                    decrytResult(result);
                }
            }
            //基于selectOne
        } else {
            if (needToDecrypt(resultObject)) {
                decrytResult(resultObject);
            }
        }
        return resultObject;
    }

    private static void decrytResult(Object result) throws IllegalAccessException {
        Class<?> aClass = result.getClass();
        Field[] allFields = ReflectUtil.getFields(aClass);
        for (Field field : allFields) {
            if (field.isAnnotationPresent(SensitiveField.class)) {
                field.setAccessible(true);
                field.set(result, Utils.isEmpty(field.get(result)) ? "" : AesUtils.decrypt(field.get(result).toString()));
            }
        }
    }

    /**
     * mybatis映射参数进行加密
     *
     * @param invocation 参数
     * @return 结果
     * @throws Throwable 异常
     */
    private Object parameterEncrypt(Invocation invocation) throws Throwable {
        //@Signature 指定了 type= parameterHandler 后，这里的 invocation.getTarget() 便是parameterHandler
        //若指定ResultSetHandler ，这里则能强转为ResultSetHandler
        ParameterHandler parameterHandler = (ParameterHandler) invocation.getTarget();
        Object parameterObject = parameterHandler.getParameterObject();

        if (parameterObject instanceof MapperMethod.ParamMap) {
            MapperMethod.ParamMap paramMap = (MapperMethod.ParamMap) parameterObject;
            parameterObject = paramMap.get("param1");
        }
        if (null == parameterObject) {
            return invocation.proceed();
        }

        if (parameterObject instanceof ArrayList) {
            ArrayList parameterList = (ArrayList) parameterObject;
            if (!CollectionUtils.isEmpty(parameterList)) {
                for (Object o : parameterList) {
                    handle(o);
                }
            }
            //基于selectOne
        } else {
            handle(parameterObject);
        }
        return invocation.proceed();
    }

    private static void handle(Object parameterObject) throws IllegalAccessException {
        if (parameterObject == null) {
            return;
        }
        Class<?> parameterObjectClass = parameterObject.getClass();
        //校验该实例的类是否被@SensitiveEntity所注解
        SensitiveData sensitiveEntity = AnnotationUtils.findAnnotation(parameterObjectClass, SensitiveData.class);
        //未被@SensitiveEntity所注解 则为null
        if (!Objects.isNull(sensitiveEntity)) {
            //取出当前当前类所有字段，传入加密方法
            //Field[] declaredFields = parameterObjectClass.getDeclaredFields();
            Field[] allFields = ReflectUtil.getFields(parameterObjectClass);
            encrytParam(allFields, parameterObject);
        }
    }

    private static void encrytParam(Field[] allFields, Object parameterObject) throws IllegalAccessException {
        for (Field field : allFields) {
            if (field.isAnnotationPresent(SensitiveField.class)) {
                field.setAccessible(true);
                field.set(parameterObject, Utils.isEmpty(field.get(parameterObject)) ? "" : AesUtils.encrypt(field.get(parameterObject).toString()));
            }
        }
    }

    /**
     * 替换mybatis Sql中的加密Key
     *
     * @param invocation 参数
     * @return 结果
     * @throws Throwable 异常
     */
    private Object replaceSql(Invocation invocation) throws Throwable {
        StatementHandler statementHandler = (StatementHandler) invocation.getTarget();
        BoundSql boundSql = statementHandler.getBoundSql();
        //获取到原始sql语句
        String sql = boundSql.getSql();
        if (null == sql) {
            return invocation.proceed();
        }
        //通过反射修改sql语句
        Field field = boundSql.getClass().getDeclaredField("sql");
        field.setAccessible(true);
        field.set(boundSql, sql);
        return invocation.proceed();
    }

    /**
     * 判断是否包含需要加解密对象
     *
     * @param object 参数
     * @return 结果
     */
    private boolean needToDecrypt(Object object) {
        if (Objects.isNull(object)) {
            return false;
        }
        Class<?> objectClass = object.getClass();
        Class<?> parentClass = objectClass.getSuperclass();
        SensitiveData sensitiveEntity = AnnotationUtils.findAnnotation(objectClass, SensitiveData.class);
        SensitiveData parentSensitiveEntity = AnnotationUtils.findAnnotation(parentClass, SensitiveData.class);

        return Objects.nonNull(sensitiveEntity) || Objects.nonNull(parentSensitiveEntity);
    }

    @Override
    public Object plugin(Object target) {
        return Plugin.wrap(target, this);
    }

    @Override
    public void setProperties(Properties properties) {

    }

}
